# frozen_string_literal: true

module GemCodebreaker
  class Game
    LENGTH_OF_SECRET_CODE = 4
    DIGIT_RANGE = (1..6).freeze

    include Validator
    attr_reader :statistic, :matcher_hash, :config

    def initialize(user, level)
      @config = GameConfig.new(user, level)
      @statistic = []
      secrete_code
    end

    def guess_start(input_code)
      guess_valid?(input_code)
      raise GemCodebreaker::NoAttemptsError if @config.attempts_used?

      @config.increase_user_attempts
      user_code = input_code.chars.map(&:to_i)
      code_marker = CodeMaker.new(@secrete_code, user_code)
      code_marker.calculate_result_code
      @matcher_hash = { plus: code_marker.plus_match, minus: code_marker.minus_match, empty: code_marker.empty_match }
    end

    def win_game?
      @matcher_hash[:plus] == LENGTH_OF_SECRET_CODE
    end

    def game_over?
      @config.attempts_used? && win_game? == false
    end

    def show_secrete_code
      @secrete_code if win_game? || game_over?
    end

    def hint
      raise GemCodebreaker::NoHintsError if @config.hints_used?

      @config.increase_user_hints
      @secrete_code.sample
    end

    def restart_game
      @config.reset_user_attempts_hints
      @secrete_code = false
      secrete_code
    end

    def save_user_statistic
      @statistic << GameStatistic.new(self)
    end

    private

    def guess_valid?(user_code)
      validate_class(user_code, String)
      raise GemCodebreaker::LengthError, I18n.t('errors.incorrect_length') if user_code.length != LENGTH_OF_SECRET_CODE
      unless user_code[/\A[#{DIGIT_RANGE.first}-#{DIGIT_RANGE.last}]{#{LENGTH_OF_SECRET_CODE}}\z/]
        raise GemCodebreaker::ValueRangeError
      end

      true
    end

    def secrete_code
      @secrete_code ||= (1..LENGTH_OF_SECRET_CODE).map { rand DIGIT_RANGE }
    end
  end
end
